@@ -7,100 +7,111 @@ use warnings;
 use Carp         'confess';
 use Scalar::Util 'blessed', 'weaken', 'looks_like_number';
 
-our $VERSION   = '0.01';
+our $VERSION   = '0.02';
 our $AUTHORITY = 'cpan:STEVAN';
 
-use base 'Class::MOP::Method';
+use base 'Class::MOP::Method::Generated';
 
 sub new {
     my $class   = shift;
     my %options = @_;
-        
-    (exists $options{options} && ref $options{options} eq 'HASH')
-        || confess "You must pass a hash of options"; 
-    
+
+    (blessed $options{metaclass} && $options{metaclass}->isa('Class::MOP::Class'))
+        || confess "You must pass a metaclass instance if you want to inline"
+            if $options{is_inline};
+
     my $self = bless {
         # from our superclass
         '&!body'          => undef,
         # specific to this subclass
-        '%!options'       => $options{options},
-        '$!meta_instance' => $options{metaclass}->get_meta_instance,
-        '@!attributes'    => [ $options{metaclass}->compute_all_applicable_attributes ], 
-        # ...
+        '%!options'              => $options{options} || {},
         '$!associated_metaclass' => $options{metaclass},
+        '$!is_inline'            => ($options{is_inline} || 0),
     } => $class;
 
-    # we don't want this creating 
-    # a cycle in the code, if not 
+    # we don't want this creating
+    # a cycle in the code, if not
     # needed
-    weaken($self->{'$!associated_metaclass'});    
+    weaken($self->{'$!associated_metaclass'});
 
-    $self->intialize_body;
+    $self->initialize_body;
 
-    return $self;    
+    return $self;
 }
 
-## predicates
-
-# NOTE:
-# if it is blessed into this class, 
-# then it is always inlined, that is 
-# pretty much what this class is for.
-sub is_inline { 1 }
-
-## accessors 
-
-sub options       { (shift)->{'%!options'}       }
-sub meta_instance { (shift)->{'$!meta_instance'} }
-sub attributes    { (shift)->{'@!attributes'}    }
+## accessors
 
+sub options              { (shift)->{'%!options'}              }
 sub associated_metaclass { (shift)->{'$!associated_metaclass'} }
 
+## cached values ...
+
+sub meta_instance {
+    my $self = shift;
+    $self->{'$!meta_instance'} ||= $self->associated_metaclass->get_meta_instance;
+}
+
+sub attributes {
+    my $self = shift;
+    $self->{'@!attributes'} ||= [ $self->associated_metaclass->compute_all_applicable_attributes ]
+}
+
 ## method
 
-sub intialize_body {
+sub initialize_body {
+    my $self        = shift;
+    my $method_name = 'generate_constructor_method';
+
+    $method_name .= '_inline' if $self->is_inline;
+
+    $self->{'&!body'} = $self->$method_name;
+}
+
+sub generate_constructor_method {
+    return sub { (shift)->meta->new_object(@_) }
+}
+
+sub generate_constructor_method_inline {
     my $self = shift;
-    # TODO:
-    # the %options should also include a both 
-    # a call 'initializer' and call 'SUPER::' 
-    # options, which should cover approx 90% 
-    # of the possible use cases (even if it 
-    # requires some adaption on the part of 
-    # the author, after all, nothing is free)
+
     my $source = 'sub {';
     $source .= "\n" . 'my ($class, %params) = @_;';
+
+    $source .= "\n" . 'return $class->meta->new_object(%params)';
+    $source .= "\n" . '    if $class ne \'' . $self->associated_metaclass->name . '\';';
+
     $source .= "\n" . 'my $instance = ' . $self->meta_instance->inline_create_instance('$class');
-    $source .= ";\n" . (join ";\n" => map { 
-        $self->_generate_slot_initializer($_) 
+    $source .= ";\n" . (join ";\n" => map {
+        $self->_generate_slot_initializer($_)
     } 0 .. (@{$self->attributes} - 1));
     $source .= ";\n" . 'return $instance';
-    $source .= ";\n" . '}'; 
-    warn $source if $self->options->{debug};   
-    
+    $source .= ";\n" . '}';
+    warn $source if $self->options->{debug};
+
     my $code;
     {
         # NOTE:
         # create the nessecary lexicals
-        # to be picked up in the eval 
+        # to be picked up in the eval
         my $attrs = $self->attributes;
-        
+
         $code = eval $source;
         confess "Could not eval the constructor :\n\n$source\n\nbecause :\n\n$@" if $@;
     }
-    $self->{'&!body'} = $code;
+    return $code;
 }
 
 sub _generate_slot_initializer {
     my $self  = shift;
     my $index = shift;
-    
+
     my $attr = $self->attributes->[$index];
-    
+
     my $default;
     if ($attr->has_default) {
         # NOTE:
         # default values can either be CODE refs
-        # in which case we need to call them. Or 
+        # in which case we need to call them. Or
         # they can be scalars (strings/numbers)
         # in which case we can just deal with them
         # in the code we eval.
@@ -114,12 +125,21 @@ sub _generate_slot_initializer {
                 $default = "'$default'";
             }
         }
+    } elsif( $attr->has_builder ) {
+        $default = '$instance->'.$attr->builder;
     }
-    $self->meta_instance->inline_set_slot_value(
-        '$instance', 
-        ("'" . $attr->name . "'"), 
-        ('$params{\'' . $attr->init_arg . '\'}' . (defined $default ? (' || ' . $default) : ''))
-    );   
+
+  'if(exists $params{\'' . $attr->init_arg . '\'}){' . "\n" .
+        $self->meta_instance->inline_set_slot_value(
+            '$instance',
+            ("'" . $attr->name . "'"),
+            '$params{\'' . $attr->init_arg . '\'}' ) . "\n" .
+   '} ' . (!defined $default ? '' : 'else {' . "\n" .
+        $self->meta_instance->inline_set_slot_value(
+            '$instance',
+            ("'" . $attr->name . "'"),
+             $default ) . "\n" .
+   '}');
 }
 
 1;
@@ -130,28 +150,28 @@ __END__
 
 =pod
 
-=head1 NAME 
+=head1 NAME
 
 Class::MOP::Method::Constructor - Method Meta Object for constructors
 
 =head1 SYNOPSIS
 
   use Class::MOP::Method::Constructor;
-  
+
   my $constructor = Class::MOP::Method::Constructor->new(
-      metaclass => $metaclass,       
+      metaclass => $metaclass,
       options   => {
           debug => 1, # this is all for now
-      },                        
+      },
   );
-  
+
   # calling the constructor ...
   $constructor->body->($metaclass->name, %params);
-  
+
 =head1 DESCRIPTION
 
-This is a subclass of C<Class::MOP::Method> which deals with 
-class constructors.  
+This is a subclass of C<Class::MOP::Method> which deals with
+class constructors.
 
 =head1 METHODS
 
@@ -169,22 +189,32 @@ This returns the metaclass which is passed into C<new>.
 
 =item B<attributes>
 
-This returns the list of attributes which are associated with the 
+This returns the list of attributes which are associated with the
 metaclass which is passed into C<new>.
 
 =item B<meta_instance>
 
-This returns the meta instance which is associated with the 
+This returns the meta instance which is associated with the
 metaclass which is passed into C<new>.
 
 =item B<is_inline>
 
-This returns a boolean, but since constructors are very rarely 
+This returns a boolean, but since constructors are very rarely
 not inlined, this always returns true for now.
 
-=item B<intialize_body>
+=item B<initialize_body>
 
-This creates the code reference for the constructor itself. 
+This creates the code reference for the constructor itself.
+
+=back
+
+=head2 Method Generators
+
+=over 4
+
+=item B<generate_constructor_method>
+
+=item B<generate_constructor_method_inline>
 
 =back
 
@@ -194,12 +224,12 @@ Stevan Little E<lt>stevan@iinteractive.comE<gt>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright 2006, 2007 by Infinity Interactive, Inc.
+Copyright 2006-2008 by Infinity Interactive, Inc.
 
 L<http://www.iinteractive.com>
 
 This library is free software; you can redistribute it and/or modify
-it under the same terms as Perl itself. 
+it under the same terms as Perl itself.
 
 =cut
 
