@@ -6,7 +6,7 @@ use warnings;
 
 use Scalar::Util 'blessed';
 
-our $VERSION   = '0.01';
+our $VERSION   = '0.02';
 our $AUTHORITY = 'cpan:STEVAN';
 
 # introspection
@@ -16,6 +16,22 @@ sub meta {
     Class::MOP::Class->initialize(blessed($_[0]) || $_[0]);
 }
 
+# RANT:
+# Cmon, how many times have you written 
+# the following code while debugging:
+# 
+#  use Data::Dumper; 
+#  warn Dumper $obj;
+#
+# It can get seriously annoying, so why 
+# not just do this ...
+sub dump { 
+    my $self = shift;
+    require Data::Dumper;
+    $Data::Dumper::Maxdepth = shift || 1;
+    Data::Dumper::Dumper $self;
+}
+
 1;
 
 __END__
@@ -50,7 +66,7 @@ and really just exists to make the Class::MOP metamodel complete.
                  
   legend:
     ..(is an instance of)..>
-    --(is a subclass of)-->
+    --(is a subclass of)--->
 
 A deeper discussion of this model is currently beyond the scope of 
 this documenation. 
@@ -61,6 +77,14 @@ this documenation.
 
 =item B<meta>
 
+=item B<dump (?$max_depth)>
+
+This will C<require> the L<Data::Dumper> module and then dump a 
+representation of your object. It passed the C<$max_depth> arg 
+to C<$Data::Dumper::Maxdepth>. The default C<$max_depth> is 1, 
+so it will not go crazy and print a massive bunch of stuff. 
+Adjust this as nessecary.
+
 =back
 
 =head1 AUTHORS
@@ -69,11 +93,11 @@ Stevan Little E<lt>stevan@iinteractive.comE<gt>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright 2006 by Infinity Interactive, Inc.
+Copyright 2006-2008 by Infinity Interactive, Inc.
 
 L<http://www.iinteractive.com>
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself.
 
-=cut
\ No newline at end of file
+=cut
